#ifndef XS_TEST_TESTER_H_INCLUDED
#define XS_TEST_TESTER_H_INCLUDED

#include <memory>
#include <unordered_map>

#include <boost/filesystem.hpp>

#include "positivegrid/exchange_server.h"
#include "test/Config.h"


namespace xs { namespace test
{
    class Tester final
    {
    public:
        struct Report
        {
            bool pass;
            std::unordered_set<std::string> missedCommands;
            std::unordered_set<std::string> missedEvents;
        };

    public:
        Tester() = delete;
        ~Tester() noexcept = default;

        Tester(const Tester&) = delete;
        Tester(Tester&&) = delete;

        Tester& operator=(const Tester&) = delete;
        Tester& operator=(Tester&&) = delete;

        explicit Tester(const boost::filesystem::path& file);

        void run() noexcept;

    private: 
        std::string onCommand(const std::string& name, const std::string& params) noexcept;
        void onEvent(const std::string& name, const std::string& params) noexcept;

        void registerServices() noexcept;
        void resetCounters() noexcept;
        Report createReport(const std::string& response, const Expectation& expectation) noexcept;

    private:
        Config config_;
        ExchangeServer server_;
        std::unordered_map<std::string, const Service*> acceptCommands_;
        std::unordered_map<std::string, const Service*> acceptEvents_;
        std::unordered_map<std::string, unsigned> receivedCommandCount_;
        std::unordered_map<std::string, unsigned> receivedEventCount_;
    };
} }

#endif
