#include "test/Config.h"

#include <boost/filesystem.hpp>
#include <nlohmann/json.hpp>

#include <fstream>
#include <iterator>
#include <memory>
#include <sstream>
#include <stdexcept>


namespace xs { namespace test {

namespace detail
{
    std::string readAsString(const boost::filesystem::path& file)
    {
        std::ifstream stream(file.string());
        if (!stream) {
            throw std::runtime_error("cannot open file: " + file.string());
        }

        std::ostringstream oss;
        oss << stream.rdbuf();

        return oss.str();
    }
}

Config::Config(const boost::filesystem::path& file)
{
    const auto object = nlohmann::json::parse(
        detail::readAsString(file)
    );

    if (!verify(object)) {
        throw std::runtime_error("invalid config content");
    }

    for (const auto element : object["mockData"]) {
        services_.emplace_back(Service(element));
    }

    for (const auto element : object["script"]) {
        expectations_.emplace_back(Expectation(element));
    }
}

boost::iterator_range<
    Config::service_range_type::const_iterator
> Config::getServices() const noexcept
{
    return boost::make_iterator_range(
        std::begin(services_), std::end(services_)
    );
}

boost::iterator_range<
    Config::expectation_range_type::const_iterator
> Config::getExpectations() const noexcept
{
    return boost::make_iterator_range(
        std::begin(expectations_), std::end(expectations_)
    );
}

bool Config::verify(const nlohmann::json& object) noexcept
{
    if (!(object.count("mockData") && object["mockData"].is_array())) {
        return false;
    }

    if (!(object.count("script") && object["script"].is_array())) {
        return false;
    }

    for (const auto element : object["mockData"]) {
        if (!Service::verify(element)) {
            return false;
        }
    }
    
    for (const auto element : object["script"]) {
        if (!Expectation::verify(element)) {
            return false;
        }
    }
 
    return true;
}

} }
