#include "test/Expectation.h"

#include <nlohmann/json.hpp>
#include <stdexcept>


namespace xs { namespace test
{
    Expectation::Expectation(const nlohmann::json& object)
    : Expectation(object["command"].get<std::string>(), object["expect"])
    {
        if (object.count("runs")) {
            for (const auto& element : object["runs"]) {
                if (!element.is_string()) {
                    throw std::runtime_error("element in \"runs\" is not string");
                }
                runs_.emplace(element.get<std::string>());
            }
        }

        if (object.count("posts")) {
            for (const auto& element : object["posts"]) {
                if (!element.is_string()) {
                    throw std::runtime_error("element in \"posts\" is not string");
                }
                posts_.emplace(element.get<std::string>());
            }
        }
    }

    bool Expectation::verify(const nlohmann::json& object) noexcept
    {
        if (!(object.is_object() 
              && object.count("command")
              && object.count("expect"))) {
            return false;
        }

        if (object.count("runs") && !object["runs"].is_array()) {
            return false;
        }

        if (object.count("posts") && !object["posts"].is_array()) {
            return false;
        }

        return (
            object["command"].is_string()
            && (
                object["expect"].is_string() || object["expect"].is_object()
            )
        );
    }

    Expectation::Expectation(std::string name, nlohmann::json value) noexcept
    : name_(std::move(name))
    , value_(std::move(value))
    { }
} }
